// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Src <string uiname="Source";>;
sampler SrcSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Src);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

// mask texture
texture Mask<string uiname="mask";>;
sampler MaskSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Mask);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//source transformation
float4x4 tTex: TEXTUREMATRIX <string uiname="Source Transform";>;

//mask transformation
float4x4 tMask: TEXTUREMATRIX <string uiname="Mask Transform";>;

//pin for the overall alpha amount
float AlphaAmount
<
    string uiname="Transparency";
    float uimin=0.0;
    float uimax=1.0;
> = 0.0;

//pin for the amount of mask to use as alpha
float MaskAlphaAmount
<
    string uiname="Mask amount";
    float uimin=0.0;
    float uimax=1.0;
> = 0.0;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
    float2 MaskCd : TEXCOORD1;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0,
    float4 maskCd : TEXCOORD1)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    //transform the mask texture cordinates
    Out.MaskCd = mul(TexCd, tMask);
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 psMask(vs2ps In): COLOR
{
    //source texture lookup
    float4 col = tex2D(SrcSamp, In.TexCd);
    // the mask texture lookup
    float4 maskcol = tex2D(MaskSamp, In.MaskCd);
    // the destination color
    float4 outColor = { 0.0, 0.0, 0.0, 1.00000 };
    //do a cheap luminance calculation
    float luminance = (0.3*maskcol.r + 0.59*maskcol.g + 0.11*maskcol.b);
    //do alinear interpolation between the mask r channel, and white
    //based on the mask amount slider
    float maskalpha = lerp(luminance, 1.0, 1-MaskAlphaAmount);
    //set the outcolor
    outColor = col;
    //generate the alpha
    float alpha = mul(col.a, maskalpha);
    //generate the final alpha based on alpha amount
    outColor.a = mul(alpha, 1-AlphaAmount);

    return outColor;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TMask
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 psMask();
    }
}

technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (SrcSamp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
